@@ -2,14 +2,28 @@
 """"Adapter to interact with the DAPI Server."""
 from __future__ import annotations
 
+import functools
 import itertools
 import json
 import time
 from collections import defaultdict
 from dataclasses import dataclass, field, fields
+from datetime import datetime
 from enum import Enum
 from importlib.metadata import version
-from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+)
 from urllib.parse import urljoin
 
 import requests
@@ -29,15 +43,22 @@ from opendapi.logging import (
     LogDistKey,
     Timer,
     increment_counter,
+    logger,
     sentry_sdk,
 )
-from opendapi.utils import make_api_w_query_and_body
+from opendapi.utils import has_underlying_model_changed, make_api_w_query_and_body
+from opendapi.validators.defs import CollectedFile
+from opendapi.writers.utils import get_writer_for_entity
 
 TOTAL_RETRIES = 5
 RETRY_BACKOFF_FACTOR = 10
 
+T = TypeVar("T")
 
-def _simple_iter_chunks(data, size=1):
+
+def _simple_iter_chunks(
+    data: Iterable[T], size: int = 1
+) -> Generator[List[T], None, None]:
     """Helper for chunking data into lists of size `size`."""
     iterator = iter(data)
     for _ in range(0, len(data), size):
@@ -72,6 +93,8 @@ class DAPIServerConfig:
     woven_integration_mode: str = None
     woven_configuration: str = None
     feature_validate_dapi_batch_size: int = 5
+    server_sync_batch_size: int = 15
+    base_generated_dbt_fallback_batch_size: int = 5
 
     @property
     def is_repo_in_shadow_mode(self) -> bool:
@@ -103,6 +126,13 @@ class DAPIServerMeta:
     portal_url: Optional[str] = None
 
 
+class CICDIntegration(Enum):
+    """Enum for CICD integrations."""
+
+    GITHUB_BUILDKITE = "github_buildkite"
+    GITHUB_GITHUB = "github_github"
+
+
 class DAPIServerRequestType(Enum):
     """Enum for DAPI Server Request Types."""
 
@@ -120,6 +150,10 @@ class DAPIServerRequestType(Enum):
     VALIDATE_FEATURES = "/v1/cicd/features/validate"
     NOTIFY = "/v1/registry/notify"
     GITHUB_PR_COMMENT = "/v1/cicd/github/pr/comment"
+    GITHUB_BUILDKITE_CICD_START = "/v1/cicd/start/github_buildkite"
+    GITHUB_GITHUB_CICD_START = "/v1/cicd/start/github_github"
+    CICD_PERSIST_FILES_BLUEPRINT = "/v1/cicd/files/persist/{integration}"
+    CICD_COMPLETE_BLUEPRINT = "/v1/cicd/complete/{integration}"
 
 
 @dataclass
@@ -403,6 +437,190 @@ class DAPIChangeNotification:
         )
 
 
+@dataclass
+class OpenDAPIEntityCICDMetadata:
+    """Metadata for OpenDAPI entities"""
+
+    entity: OpenDAPIEntity
+    # NOTE: make sure this is formatted for the server!
+    filepath: str
+    base_collect: CollectedFile | None
+    head_collect: CollectedFile | None
+    integration_mode: str
+
+    @functools.cached_property
+    def entity_changed_from_base(self) -> bool:
+        """Check if the model is changed from the model state at base."""
+        # for a non-dapi file, any change to the **file** state from base
+        # is considered a change
+        if self.entity is not OpenDAPIEntity.DAPI:
+            return (
+                self.base_collect and self.base_collect.original
+            ) != self.head_collect.merged
+
+        # we note that the file can remain after the entity is deleted
+        # in that case any file differences arent actual entity changes
+        base_generated = self.base_collect.generated if self.base_collect else None
+        head_generated = self.head_collect.generated if self.head_collect else None
+        if not base_generated and not head_generated:
+            return False
+
+        # if it exists in one but not the other, then surely it was changed
+        if bool(base_generated) != bool(head_generated):
+            return True
+
+        # entity exists at both, so we compare if there were any meaningful changes.
+        # note that we compare merged - but prune it - so that nullability
+        # changes are considered changes
+        base_merged = self.base_collect.merged if self.base_collect else None
+        head_merged = self.head_collect.merged if self.head_collect else None
+
+        return has_underlying_model_changed(base_merged, head_merged)
+
+    @property
+    def entity_is_new(self) -> bool:
+        """Check if the model is new"""
+        # we do these gymnastics since the dapi file can remain after the entity
+        # is deleted
+        base_generated = self.base_collect.generated if self.base_collect else None
+        return self.entity_changed_from_base and not base_generated
+
+    @property
+    def entity_is_deprecated(self) -> bool:
+        """Check if the model is deprecated"""
+        head_generated = self.head_collect.generated if self.head_collect else None
+        return self.entity_changed_from_base and not head_generated
+
+    # NOTE: entity cannot be changed from head, since we do not
+    #       ever change anything at the ORM level
+
+    @property
+    def file_git_tracked(self) -> bool:
+        """
+        Check if the file is tracked by git
+        determined if the file ever existed
+        """
+        return (
+            self.head_collect is not None and self.head_collect.original is not None
+        ) or (self.base_collect is not None and self.base_collect.original is not None)
+
+    @functools.cached_property
+    def _should_be_processed(self) -> tuple[bool, str]:
+        """Check if the entity should be processed"""
+        # in shadow mode, we only run CICD if the dapi was onboarded
+        # or is being onboarded
+        # TODO note about possibly doing something else for enrich during shadow for a diff experience
+        in_shadow_mode = self.integration_mode == "shadow"
+        if in_shadow_mode and not self.file_git_tracked:
+            return False, "SHADOW_MODE_AND_FILE_NOT_TRACKED"
+
+        # the entity and file are not at head (therefore at least one is at base).
+        # this is because either the model was just deleted, or both
+        # the model and the dapi were just deleted. regardless,
+        # there is nothing to check
+        if not self.head_collect:
+            return False, "ENTITY_AND_FILE_NOT_AT_HEAD"
+
+        # first we handle non DAPI cases
+        # any file changes from be processed
+        if self.entity is not OpenDAPIEntity.DAPI:
+            # if the **file** didnt exist and now exists in any capacity,
+            # we need to process.
+            # note that we consider file changes, since we **always** generate
+            # a single opendapi file if there arent any for non dapis
+            if (
+                not (self.base_collect and self.base_collect.original)
+                and self.head_collect.merged
+            ):
+                return True, "NON_DAPI_FILE_CREATED"
+
+            # if the file changed from base, we need to process
+            if (
+                self.base_collect and self.base_collect.original
+            ) != self.head_collect.merged:
+                return True, "NON_DAPI_FILE_CHANGED_FROM_BASE"
+
+            # if the file changed from head, we need to process
+            if self.head_collect.merged != self.head_collect.original:
+                return True, "NON_DAPI_FILE_CHANGED_FROM_HEAD"
+
+            return False, "NON_DAPI_FILE_NO_CHANGE"
+
+        # we consider if the model itself still exists
+        if self.head_collect.generated:
+            # there was a meaningful change to the model.
+            # we need to process
+            if self.entity_changed_from_base:
+                return True, "MODEL_EXISTS_AND_ENTITY_CHANGED"
+
+            # there was no meaningful change to the model, but
+            # there is no dapi. we need to create one.
+            if not self.head_collect.original:
+                return True, "MODEL_EXISTS_AND_NO_FILE"
+
+            # there was no meaningful change to the model, the dapi exists,
+            # but the dapi materially differs from the model, meaning it is out of sync.
+            # note that this only applies to actual models.
+            # we need to process
+            # note that we compare against merged, since we want to respect nullability changes
+            if has_underlying_model_changed(
+                self.head_collect.merged, self.head_collect.original
+            ):
+                return True, "MODEL_EXISTS_AND_FILE_OUT_OF_SYNC"
+
+            # no meaningful change to the model and the dapi is synced.
+            # in this scenario, we just need to make sure that manual
+            # changes are sane.
+            # either the file was just added - which is the case above - or
+            # the file was manually changed, in both cases we need to process
+            if self.head_collect.original != (
+                self.base_collect and self.base_collect.original
+            ):
+                return True, "MODEL_EXISTS_AND_FILE_CHANGE"
+
+            return False, "MODEL_EXISTS_AND_NO_CHANGE"
+
+        # now we consider the file existing but the model not existing.
+        # if there were any changes to the file, we need to process
+        # either the entity was just deleted but not the file, the file is a vestige and changed,
+        # or the file was manually added
+        if self.head_collect.original != (
+            self.base_collect and self.base_collect.original
+        ):
+            return True, "MODEL_DOESNT_EXIST_FILE_EXISTS_AND_CHANGED"
+
+        return False, "MODEL_DOESNT_EXIST_FILE_EXISTS_AND_NO_CHANGE"
+
+    @functools.cached_property
+    def should_be_processed(self) -> tuple[bool, str]:
+        """Check if the entity should be processed"""
+        result, _ = self._should_be_processed
+        return result
+
+    @property
+    def should_be_processed_reason(self) -> str:
+        """Get the reason for processing the entity."""
+        _, reason = self._should_be_processed
+        return reason
+
+    @property
+    def for_server(self) -> dict:
+        """Convert to dict."""
+        return {
+            "entity": self.entity.value,
+            "filepath": self.filepath,
+            "base_collect": self.base_collect.for_server if self.base_collect else None,
+            "head_collect": self.head_collect.for_server if self.head_collect else None,
+            "entity_changed_from_base": self.entity_changed_from_base,
+            "entity_is_new": self.entity_is_new,
+            "entity_is_deprecated": self.entity_is_deprecated,
+            "file_git_tracked": self.file_git_tracked,
+            "integration_mode": self.integration_mode,
+            "should_be_processed": self.should_be_processed,
+            "should_be_processed_reason": self.should_be_processed_reason,
+        }
+
+
 @dataclass
 class DAPIServerResponse:
     """DAPI server Response formatted"""
@@ -561,7 +779,7 @@ class DAPIRequests:
     @staticmethod
     def _construct_schemas_from_dict(schemas_dict: dict) -> Schemas:
         """Construct Schemas from a dictionary."""
-        return Schemas(
+        return Schemas.create(
             teams=schemas_dict.get("teams"),
             datastores=schemas_dict.get("datastores"),
             purposes=schemas_dict.get("purposes"),
@@ -650,6 +868,7 @@ class DAPIRequests:
                 "repo_html_url": trigger_event.repo_html_url,
                 "pull_request_number": trigger_event.pull_request_number,
                 "pull_request_link": trigger_event.pull_request_link,
+                "branch": trigger_event.branch,
             },
         }
 
@@ -1510,3 +1729,161 @@ class DAPIRequests:
             self._handle_api_error(
                 DAPIServerRequestType.GITHUB_PR_COMMENT.value, response.status_code
             )
+
+    def _github_repo_cicd_start(
+        self,
+        request_path: str,
+        runner_params: Dict[str, Any],
+    ) -> str:
+        """
+        Returns a common payload for all github repo operations
+        """
+
+        response, _ = self.raw_send_request_to_dapi_server(
+            request_path=request_path,
+            query_params={
+                "repo_name": self.trigger_event.repo_full_name,
+            },
+            body_json={
+                "event": self.trigger_event.event_type,
+                "branch": self.trigger_event.branch,
+                "base_commit_sha": self.trigger_event.before_change_sha,
+                "head_commit_sha": self.trigger_event.after_change_sha,
+                "head_commit_sha_timestamp": self.trigger_event.after_change_sha_timestamp,
+                "pr_link": self.trigger_event.pull_request_link,
+                "version": f"opendapi-{version('opendapi')}",
+                **runner_params,
+            },
+            method=HTTPMethod.POST,
+        )
+
+        if response.status_code >= 400:
+            self._handle_api_error(
+                request_path,
+                response.status_code,
+            )
+
+        return response.json()["woven_cicd_id"]
+
+    def cicd_start_github_buildkite(
+        self,
+        job_id: str,
+        job_started_at: datetime,
+        build_id: str,
+        retry_count: Optional[int] = None,
+    ) -> str:
+        """Notify the DAPI Server that the GitHub CI/CD has started."""
+        return self._github_repo_cicd_start(
+            DAPIServerRequestType.GITHUB_BUILDKITE_CICD_START.value,
+            {
+                "job_id": job_id,
+                "job_started_at": job_started_at.isoformat(),
+                "build_id": build_id,
+                "retry_count": retry_count,
+            },
+        )
+
+    def cicd_start_github_github(
+        self,
+        run_id: str,
+        run_attempt: int,
+        run_number: int,
+    ) -> str:
+        """Notify the DAPI Server that the GitHub CI/CD has started."""
+        return self._github_repo_cicd_start(
+            DAPIServerRequestType.GITHUB_GITHUB_CICD_START.value,
+            {
+                "run_id": run_id,
+                "run_attempt": run_attempt,
+                "run_number": run_number,
+            },
+        )
+
+    def cicd_persist_files(
+        self,
+        integration: CICDIntegration,
+        base_collected_files: Dict[OpenDAPIEntity, Dict[str, CollectedFile]],
+        head_collected_files: Dict[OpenDAPIEntity, Dict[str, CollectedFile]],
+        woven_cicd_id: str,
+        notify_function: Optional[Callable[[int], None]] = None,
+    ) -> Dict[OpenDAPIEntity, List[str]]:
+        """Persist the CI/CD files with the DAPI Server."""
+        # first, create the CICDMetadata objects
+        total_entities = base_collected_files.keys() | head_collected_files.keys()
+        opendapi_file_metadata: List[OpenDAPIEntityCICDMetadata] = []
+        entity_to_filepath = defaultdict(list)
+        for entity in total_entities:
+            bc_raw = base_collected_files.get(entity, {})
+            bc = {
+                collected_file.server_filepath: collected_file
+                for collected_file in bc_raw.values()
+            }
+            hc_raw = head_collected_files.get(entity, {})
+            hc = {
+                collected_file.server_filepath: collected_file
+                for collected_file in hc_raw.values()
+            }
+            total_filepaths = bc.keys() | hc.keys()
+            for filepath in total_filepaths:
+                entity_to_filepath[entity].append(filepath)
+                opendapi_file_metadata.append(
+                    OpenDAPIEntityCICDMetadata(
+                        entity=entity,
+                        filepath=filepath,
+                        base_collect=bc.get(filepath),
+                        head_collect=hc.get(filepath),
+                        integration_mode=self.dapi_server_config.woven_integration_mode,
+                    )
+                )
+
+        request_path = DAPIServerRequestType.CICD_PERSIST_FILES_BLUEPRINT.value.format(
+            integration=integration.value
+        )
+        for batch in _simple_iter_chunks(
+            opendapi_file_metadata, self.dapi_server_config.server_sync_batch_size
+        ):
+            response, _ = self.raw_send_request_to_dapi_server(
+                request_path=request_path,
+                query_params={
+                    "woven_cicd_id": woven_cicd_id,
+                },
+                body_json={
+                    "opendapi_files": [
+                        opendapi_file_metadata.for_server
+                        for opendapi_file_metadata in batch
+                    ],
+                },
+                method=HTTPMethod.POST,
+            )
+
+            if response.status_code >= 400:
+                self._handle_api_error(request_path, response.status_code)
+
+            if notify_function:
+                notify_function(len(batch))
+
+        return entity_to_filepath
+
+    def cicd_complete(
+        self,
+        integration: CICDIntegration,
+        metadata_file: Dict[str, Any],
+        woven_cicd_id: str,
+    ) -> None:
+        """Notify the DAPI Server that the CI/CD has completed."""
+        request_path = DAPIServerRequestType.CICD_COMPLETE_BLUEPRINT.value.format(
+            integration=integration.value
+        )
+        response, _ = self.raw_send_request_to_dapi_server(
+            request_path=request_path,
+            query_params={
+                "woven_cicd_id": woven_cicd_id,
+            },
+            body_json={
+                "metadata": metadata_file,
+                "logs": logger.get_logs(),
+            },
+            method=HTTPMethod.POST,
+        )
+        if response.status_code >= 400:
+            self._handle_api_error(request_path, response.status_code)
