@@ -3,14 +3,10 @@
 import click
 import sentry_sdk
 
-from opendapi.cli.generate import cli as generate_cli
 from opendapi.cli.init import cli as init_cli
 from opendapi.cli.repos.github.main import cli as github_repo_cli
 from opendapi.cli.repos.local.main import cli as local_repo_cli
 
-# NOTE: this is for backwards compat, remove after migrated
-from opendapi.cli.run import cli as run_cli
-
 
 @click.group()
 def cli():
@@ -38,13 +34,11 @@ def cli_wrapper():
         # these to sentry. However, if the application fails due to an internal
         # error, we will catch it and log it.
         sentry_sdk.capture_exception(exp)
+        sentry_sdk.flush()
         raise exp
 
 
 # Add commands to the CLI
 cli.add_command(init_cli, name="init")
-cli.add_command(generate_cli, name="generate")
 cli.add_command(github_repo_cli, name="github")
 cli.add_command(local_repo_cli, name="local")
-# NOTE: this is for backwards compat, remove after migrated
-cli.add_command(run_cli, name="run")
