@@ -14,11 +14,15 @@ from opendapi.cli.options import (
     opendapi_run_options,
 )
 from opendapi.cli.repos.local.runners.local.enrich import cli as enrich_cli
+from opendapi.cli.repos.local.runners.local.generate import cli as generate_cli
 from opendapi.cli.repos.local.runners.local.options import (
     construct_change_trigger_event,
 )
 from opendapi.cli.repos.local.runners.local.register import cli as register_cli
 from opendapi.cli.repos.local.runners.local.run import cli as run_cli
+from opendapi.cli.repos.local.runners.local.split_generate import (
+    cli as split_generate_cli,
+)
 
 
 @click.group()
@@ -45,5 +49,7 @@ def cli(**kwargs):
 
 
 cli.add_command(enrich_cli, name="enrich")
+cli.add_command(generate_cli, name="generate")
 cli.add_command(register_cli, name="register")
 cli.add_command(run_cli, name="run")
+cli.add_command(split_generate_cli, name="split-generate")
