@@ -10,6 +10,7 @@ class FeatureFlag(Enum):
 
     DBT_WRITEBACK_ENABLED = "dbt_writeback_enabled"
     FEATURE_VALIDATION_ENABLED = "feature_validation_enabled"
+    PERFORM_COMPLETE_SERVER_SIDE_CICD = "perform_complete_server_side_cicd"
 
     @property
     def to_env_var(self):
@@ -33,7 +34,7 @@ def set_feature_flags(
         os.environ[flag.to_env_var] = str(value).lower()
 
 
-def get_feature_flag(flag: FeatureFlag):
+def get_feature_flag(flag: FeatureFlag) -> bool:
     """
     Get the feature flag for the OpenDAPI client.
     """
