@@ -6,10 +6,13 @@ import base64
 import importlib
 import inspect
 import io
-import json
 import os
 import re
 from copy import deepcopy
+from json import dump as json_dump
+from json import dumps as json_dumps
+from json import load as json_load
+from json import loads as json_loads
 from typing import Any, Dict, List, Optional, TextIO, Tuple
 
 import jsonref
@@ -20,7 +23,7 @@ from jsonschema.validators import validator_for
 from ruamel.yaml import YAML as _YAML
 from ruamel.yaml import CommentedMap
 
-from opendapi.defs import OPENDAPI_URL, HTTPMethod
+from opendapi.defs import DAPI_ORM_EXTRACTED_FIELDS_SCHEMA, OPENDAPI_URL, HTTPMethod
 from opendapi.logging import logger
 
 session = requests_cache.CachedSession(
@@ -136,7 +139,7 @@ def read_yaml_or_json(filepath: str, yaml: YAML = None) -> dict:
         if filepath.endswith(".yaml") or filepath.endswith(".yml"):
             return yaml.load(filepath_handle.read())
         if filepath.endswith(".json"):
-            return json.load(filepath_handle)
+            return json_load(filepath_handle)
     raise ValueError(f"Unsupported filepath type for {filepath}")
 
 
@@ -148,7 +151,7 @@ def _write_to_io(
         # this mutates the data, so we deepcopy it
         sorted_yaml_dump(deepcopy(data), io_, json_spec=json_spec, yaml=yaml)
     elif filepath.endswith(".json"):
-        json.dump(data, io_, indent=4)
+        json_dump(data, io_, indent=4)
     else:
         raise ValueError(f"Unsupported filepath type for {filepath}")
 
@@ -373,57 +376,12 @@ def prune_additional_properties(inst: dict, schema: dict) -> dict:
 
 def encode_json_to_base64(json_data: Any) -> str:
     """Encode json to a base64 string"""
-    return base64.b64encode(json.dumps(json_data).encode("utf-8")).decode("utf-8")
+    return base64.b64encode(json_dumps(json_data).encode("utf-8")).decode("utf-8")
 
 
 def decode_base64_to_json(base64_str: str) -> Any:
     """Decode a base64 string to json"""
-    return json.loads(base64.b64decode(base64_str.encode("utf-8")).decode("utf-8"))
-
-
-_ORM_EXTRACTED_FIELDS_SCHEMA = {
-    "$schema": "https://json-schema.org/draft/2020-12/schema",
-    "$defs": {
-        "field": {
-            "type": "object",
-            "properties": {
-                "name": {
-                    "type": "string",
-                },
-                "data_type": {
-                    "type": "string",
-                },
-                "is_nullable": {
-                    "type": "boolean",
-                },
-            },
-            "required": ["name", "data_type"],
-        }
-    },
-    "type": "object",
-    "properties": {
-        "schema": {
-            "type": "string",
-            "format": "uri",
-        },
-        "urn": {
-            "type": "string",
-            "pattern": "^([\\w-]+\\.)+[\\w-]+$",
-        },
-        "fields": {
-            "type": "array",
-            "items": {"$ref": "#/$defs/field"},
-            "minItems": 1,
-        },
-        "primary_key": {
-            "type": "array",
-            "items": {
-                "type": "string",
-            },
-        },
-    },
-    "required": ["schema", "urn", "fields", "primary_key"],
-}
+    return json_loads(base64.b64decode(base64_str.encode("utf-8")).decode("utf-8"))
 
 
 def has_underlying_model_changed(
@@ -435,5 +393,18 @@ def has_underlying_model_changed(
     This is done by pruning all non-ORM-derived portions from the Dapis and comparing them.
     """
     return prune_additional_properties(
-        dapi_one or {}, _ORM_EXTRACTED_FIELDS_SCHEMA
-    ) != prune_additional_properties(dapi_two or {}, _ORM_EXTRACTED_FIELDS_SCHEMA)
+        dapi_one or {}, DAPI_ORM_EXTRACTED_FIELDS_SCHEMA
+    ) != prune_additional_properties(dapi_two or {}, DAPI_ORM_EXTRACTED_FIELDS_SCHEMA)
+
+
+def build_location_without_repo_from_fullpath(root_dir: str, fullpath: str):
+    """
+    Build a full path from the qualified location.
+
+    full path is root_dir/path/to/file
+    example:
+    root_dir = /tmp/repo
+    fullpath = /tmp/repo/path/to/file
+    return path/to/file
+    """
+    return re.sub(root_dir, "", fullpath).lstrip("/")
