@@ -31,6 +31,7 @@ from opendapi.utils import (
 from opendapi.validators.defs import (
     CollectedFile,
     FileSet,
+    IntegrationType,
     MergeKeyCompositeIDParams,
     MultiValidationError,
     ValidationError,
@@ -45,7 +46,7 @@ class BaseValidator(ABC):
     # Paths & keys to use for uniqueness check within a list of dicts when merging
     MERGE_UNIQUE_LOOKUP_KEYS: List[
         Tuple[
-            List[Union[str, int, MergeKeyCompositeIDParams.IgnoreListIndex]],
+            List[Union[str, int, MergeKeyCompositeIDParams.IgnoreListIndexType]],
             MergeKeyCompositeIDParams,
         ]
     ] = []
@@ -55,9 +56,11 @@ class BaseValidator(ABC):
 
     SPEC_VERSION: str = NotImplemented
     ENTITY: OpenDAPIEntity = NotImplemented
+    INTEGRATION_TYPE: IntegrationType = IntegrationType.STATIC
 
     def __init__(
         self,
+        *,
         root_dir: str,
         enforce_existence_at: Optional[FileSet] = None,
         override_config: Optional[OpenDAPIConfig] = None,
@@ -82,7 +85,23 @@ class BaseValidator(ABC):
         """Get the base directory for the spec files"""
         return os.path.join(self.root_dir, DEFAULT_DAPIS_DIR)
 
-    def _get_merger(self):
+    @classmethod
+    def merge(cls, base: Dict, nxt: Dict) -> Dict:
+        """Merge the base and next dictionaries"""
+        return cls._get_merger().merge(deepcopy(base), deepcopy(nxt))
+
+    def get_file_state(self, fileset: FileSet) -> Dict[str, Dict]:
+        """Get the file state at a given file set"""
+        if fileset is FileSet.ORIGINAL:
+            return self.original_file_state
+        if fileset is FileSet.GENERATED:
+            return self.generated_file_state
+        if fileset is FileSet.MERGED:
+            return self.merged_file_state
+        raise ValueError(f"Invalid file set: {fileset}")  # pragma: no cover
+
+    @classmethod
+    def _get_merger(cls):
         """
         Get the merger object for deepmerge
 
@@ -129,7 +148,7 @@ class BaseValidator(ABC):
 
             result = []
             autoupdate_unique_merge_keys_for_path = [
-                v for k, v in self.MERGE_UNIQUE_LOOKUP_KEYS if k == path
+                v for k, v in cls.MERGE_UNIQUE_LOOKUP_KEYS if k == path
             ]
 
             # lets shy away from automatically deduping, and only scrub the
@@ -157,7 +176,7 @@ class BaseValidator(ABC):
             # sometimes we only want what is in base (merged) - i.e. for fields, we only want the
             # fields that are from the ORM. In that instance, just return the merged result
             # using only the keys present in base
-            if path in self.MERGE_DISALLOW_NEW_ENTRIES_PATH:
+            if path in cls.MERGE_DISALLOW_NEW_ENTRIES_PATH:
                 return result
 
             present_ids = {
@@ -194,7 +213,7 @@ class BaseValidator(ABC):
             ["override"],
         )
 
-    def _get_files_for_suffix(self, suffixes: List[str]):
+    def _get_files_for_suffix(self, suffixes: List[str]) -> List[str]:
         """Get all files in the root directory with given suffixes"""
         all_files = find_files_with_suffix(self.root_dir, suffixes)
         return [
@@ -268,6 +287,12 @@ class BaseValidator(ABC):
             error_message = f"Validation error for {file}: \n{str(exc)}"
             raise ValidationError(error_message) from exc
 
+    def _get_additional_metadata_from_generated(  # pylint: disable=unused-argument
+        self, filepath: str
+    ) -> Optional[Dict]:
+        """Get additional metadata from the generated step"""
+        return None
+
     @abstractmethod
     def _get_base_generated_files(self) -> Dict[str, Dict]:
         """
@@ -319,20 +344,14 @@ class BaseValidator(ABC):
                 merged_content = original_content
 
             else:
-                merged_content = self._get_merger().merge(
-                    deepcopy(generated_content), deepcopy(original_content)
-                )
+                merged_content = self.merge(generated_content, original_content)
 
             merged_file_state[file] = merged_content
 
         return merged_file_state
 
-    def custom_content_validations(self, file: str, content: Dict):
-        """Custom content validations if any desired"""
-
-    def validate_content(self, file: str, content: Dict):
+    def validate_content(self, file: str, content: Dict, fileset: FileSet):
         """Validate the content of the files"""
-        self.custom_content_validations(file, content)
 
     @property
     def base_tags(self) -> Dict:
@@ -342,9 +361,7 @@ class BaseValidator(ABC):
             "org_name": self.config.org_name_snakecase,
         }
 
-    def _collect_validation_errors(
-        self, file_to_content: Dict[str, Dict], fileset: FileSet
-    ) -> List[str]:
+    def _collect_validation_errors(self, fileset: FileSet) -> List[str]:
         """Run the validators"""
         # Update the files after autoupdate
         # NOTE: think about if we want to use the minimal schema to validate
@@ -356,6 +373,8 @@ class BaseValidator(ABC):
         if self.enforce_existence_at:
             self.validate_existance_at()
 
+        file_to_content = self.get_file_state(fileset)
+
         # Collect the errors for all the files
         errors = []
         for file, content in file_to_content.items():
@@ -365,9 +384,10 @@ class BaseValidator(ABC):
                 errors.append(str(exc))
             else:
                 try:
-                    self.validate_content(file, content)
+                    self.validate_content(file, content, fileset)
                 except ValidationError as exc:
-                    errors.append(str(exc))
+                    commit = self.commit_sha or "CURRENT_STATE"
+                    errors.append(f"{commit} {fileset.value}: {str(exc)}")
 
         # Increment the counter for the number of items validated
         tags = {
@@ -386,17 +406,9 @@ class BaseValidator(ABC):
         """
         All of the validation errors
         """
-        errors = self._collect_validation_errors(
-            self.original_file_state, FileSet.ORIGINAL
-        )
-        errors.extend(
-            self._collect_validation_errors(
-                self.generated_file_state, FileSet.GENERATED
-            )
-        )
-        errors.extend(
-            self._collect_validation_errors(self.merged_file_state, FileSet.MERGED)
-        )
+        errors = self._collect_validation_errors(FileSet.ORIGINAL)
+        errors.extend(self._collect_validation_errors(FileSet.GENERATED))
+        errors.extend(self._collect_validation_errors(FileSet.MERGED))
         return errors
 
     def validate(self):
@@ -434,6 +446,10 @@ class BaseValidator(ABC):
                 filepath=file,
                 commit_sha=self.commit_sha,
                 entity=self.ENTITY,
+                root_dir=self.root_dir,
+                additional_metadata_from_generated=self._get_additional_metadata_from_generated(
+                    file
+                ),
             )
             for file in self.total_filepaths
         }
@@ -453,7 +469,7 @@ class BaseValidator(ABC):
         return organized_results
 
     @classmethod
-    def run_validators(
+    def run_validators(  # pylint: disable=too-many-arguments
         cls,
         validators: Iterable[Type[BaseValidator]],
         root_dir: str,
@@ -461,6 +477,9 @@ class BaseValidator(ABC):
         override_config: Optional[OpenDAPIConfig] = None,
         minimal_schemas: Optional[Schemas] = None,
         commit_sha: Optional[str] = None,
+        # integration specific flags
+        runtime_skip_generation: bool = False,
+        dbt_skip_generation: bool = False,
     ) -> Tuple[
         Dict[OpenDAPIEntity, Dict[str, CollectedFile]], List[MultiValidationError]
     ]:
@@ -471,17 +490,24 @@ class BaseValidator(ABC):
         collected_files = []
         errors = []
         for validator in validators:
-            inst = validator(
-                root_dir=root_dir,
-                enforce_existence_at=enforce_existence_at,
-                override_config=override_config,
-                schema_to_prune_generated=(
+            kwargs = {
+                "root_dir": root_dir,
+                "enforce_existence_at": enforce_existence_at,
+                "override_config": override_config,
+                "schema_to_prune_generated": (
                     minimal_schemas.minimal_schema_for(validator)
                     if minimal_schemas
                     else None
                 ),
-                commit_sha=commit_sha,
-            )
+                "commit_sha": commit_sha,
+            }
+
+            if validator.INTEGRATION_TYPE is IntegrationType.RUNTIME:
+                kwargs["skip_generation"] = runtime_skip_generation
+            elif validator.INTEGRATION_TYPE is IntegrationType.DBT:
+                kwargs["skip_generation"] = dbt_skip_generation
+
+            inst = validator(**kwargs)
             try:
                 collected_files.extend(inst.get_validated_files().values())
             except MultiValidationError as e:
