@@ -470,8 +470,8 @@ class ActiveRecordDapiValidator(DapiValidator):
                             ),
                         },
                         "privacy_requirements": {
-                            "dsar_access_endpoint": None,
-                            "dsar_deletion_endpoint": None,
+                            "dsr_access_endpoint": None,
+                            "dsr_deletion_endpoint": None,
                         },
                     }
         return result
