@@ -7,7 +7,7 @@ import re
 from dataclasses import dataclass, field
 from enum import Enum
 from functools import cached_property, lru_cache
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Tuple
 
 from opendapi.config import (
     construct_dapi_source_sink_from_playbooks,
@@ -24,6 +24,7 @@ from opendapi.utils import find_files_with_suffix, read_yaml_or_json
 from opendapi.validators.dapi.base import DapiValidator
 from opendapi.validators.dapi.mixins import DBTCloudMixin
 from opendapi.validators.dapi.models import ProjectInfo
+from opendapi.validators.defs import IntegrationType
 
 DBT_CONFIG_YML = "dbt_project.yml"
 DBT_ARTIFACTS_DIR = "target"
@@ -270,13 +271,19 @@ class DBTModel:
                 column_names.append(column.name.lower())
         return column_names
 
-    def reconcile_custom_target_schema(self, target_schema: str) -> str:
+    @property
+    def custom_schema(self) -> Optional[str]:
+        """Get the custom schema"""
+        return self._property_from_manifest("config").get("schema")
+
+    @staticmethod
+    def reconcile_custom_target_schema(
+        target_schema: str, custom_schema: Optional[str], current_schema: str
+    ) -> str:
         """
         Figure out production schema from the CI/dev manifest and config
         https://docs.getdbt.com/docs/build/custom-schemas
         """
-        custom_schema = self._property_from_manifest("config").get("schema")
-        current_schema = self.schema
         production_target_schema = target_schema
         if custom_schema:
             if not current_schema.endswith(custom_schema):
@@ -290,13 +297,13 @@ class DBTModel:
                     # replace target schema to production env
                     # but keep the non-word connector, typically underscore.
                     # staging_marketing -> production_marketing
-                    return self.schema.replace(
+                    return current_schema.replace(
                         current_schema_prefix,
                         f"{target_schema}{non_word_suffix.group(0)}",
                     )
             # target schema is not prefixed to the custom schema according to DBT setup.
             # marketing -> marketing
-            return self.schema
+            return current_schema
         # No custom schema override so just use the target schema.
         # staging -> production
         return target_schema
@@ -305,6 +312,27 @@ class DBTModel:
         """Construct the urn"""
         return f"{self.project.org_name_snakecase}.dbt.{self.project.name}.{self.name}"
 
+    @classmethod
+    def perform_reconciliation_for_datastore_sources(
+        cls, sources: List[Dict], custom_schema: Optional[str], current_schema: str
+    ) -> Dict:
+        """Perform reconciliation for datastores"""
+        for source in sources:
+            namespace = source.get("data", {}).get("namespace")
+            if namespace:
+                # some datastores don't have a db.schema namespace but just a schema
+                if "." in namespace:
+                    database, schema = namespace.split(".")[:2]
+                    reconciled = cls.reconcile_custom_target_schema(
+                        schema, custom_schema, current_schema
+                    )
+                    source["data"]["namespace"] = f"{database}.{reconciled}"
+                else:
+                    source["data"]["namespace"] = cls.reconcile_custom_target_schema(
+                        namespace, custom_schema, current_schema
+                    )
+        return sources
+
     def construct_datastores(self) -> Dict:
         """Construct the datastores"""
         datastores = (
@@ -314,19 +342,9 @@ class DBTModel:
             if self.project.override.playbooks
             else {"sources": [], "sinks": []}
         )
-        for source in datastores["sources"]:
-            namespace = source.get("data", {}).get("namespace")
-            if namespace:
-                # some datastores don't have a db.schema namespace but just a schema
-                if "." in namespace:
-                    database, schema = namespace.split(".")[:2]
-                    source["data"][
-                        "namespace"
-                    ] = f"{database}.{self.reconcile_custom_target_schema(schema)}"
-                else:
-                    source["data"]["namespace"] = self.reconcile_custom_target_schema(
-                        namespace
-                    )
+        self.perform_reconciliation_for_datastore_sources(
+            datastores["sources"], self.custom_schema, self.schema
+        )
         return DapiValidator.add_non_playbook_datastore_fields(datastores)
 
     def construct_owner_team_urn(self) -> Optional[str]:
@@ -340,25 +358,22 @@ class DBTModel:
         )
 
 
-class DbtDapiValidator(DapiValidator, DBTCloudMixin):
+class DbtDapiValidator(DapiValidator[DBTProjectInfo], DBTCloudMixin):
     """
     Validator class for DAPIs created from DBT models
 
     """
 
     INTEGRATION_NAME = "dbt"
+    INTEGRATION_TYPE = IntegrationType.DBT
 
-    def __init__(self, *args, **kwargs):
-        self._dbt_commit_sha = kwargs.get("commit_sha") or os.environ.get(
-            "GITHUB_HEAD_SHA"
+    def __init__(self, *, skip_generation, **kwargs):
+        self._skip_generation = skip_generation
+        head_sha = os.environ.get("GITHUB_HEAD_SHA") or os.environ.get(
+            "BUILDKITE_COMMIT"
         )
-        # if dbt docs are not guaranteed/expected to be present, we can
-        # fallback to the state present at the commit sha
-        # HACK: envvar set at run skip condition
-        self._skip = (self._dbt_commit_sha != os.environ.get("GITHUB_HEAD_SHA")) or (
-            os.environ.get("ALWAYS_SKIP_DBT_SYNC") == "true"
-        )
-        super().__init__(*args, **kwargs)
+        self._dbt_commit_sha = kwargs.get("commit_sha") or head_sha
+        super().__init__(**kwargs)
 
     def get_all_dbt_config_files(self) -> List[str]:
         """Get all the dbt config files"""
@@ -416,7 +431,7 @@ class DbtDapiValidator(DapiValidator, DBTCloudMixin):
 
     def _sync_from_external_sources(self, projects: List[DBTProjectInfo]) -> None:
         """This function will sync the dbt projects from external sources"""
-        print("DBT: About to check external sources for artifacts")
+        logger.info("DBT: About to check external sources for artifacts")
         # We will run through all the external sync functions, in order, till any function
         # returns True
         self.sync_dbt_cloud_artifacts(projects)
@@ -425,7 +440,7 @@ class DbtDapiValidator(DapiValidator, DBTCloudMixin):
 
     def _assert_necessary_files_exist(self, projects: List[DBTProjectInfo]) -> None:
         """Assert that the necessary files exist"""
-        if self._skip:
+        if self._skip_generation:
             return
 
         errors = []
@@ -479,9 +494,9 @@ class DbtDapiValidator(DapiValidator, DBTCloudMixin):
 
         return projects
 
-    def _get_dbt_models(self) -> List[Dict]:
+    def _get_dbt_models(self) -> List[DBTModel]:
         """Get the DBT models from manifest.json and enrich with catalog.json"""
-        if self._skip:
+        if self._skip_generation:
             raise RuntimeError("Cannot get DBT models when skipping artifact downloads")
 
         dbt_models = []
@@ -521,26 +536,31 @@ class DbtDapiValidator(DapiValidator, DBTCloudMixin):
             )
         return dbt_models
 
-    def _get_base_generated_files(self) -> Dict[str, Dict]:
+    @cached_property
+    def _base_generated_files_and_generated_metadata(
+        self,
+    ) -> Tuple[Dict[str, Dict], Dict[str, Optional[Dict]]]:
         """Get the base generated files"""
 
         # if we need to skip, just fallback to the original file state
-        if self._skip:
-            print("DBT: skipping source sync, defaulting to original file state")
+        if self._skip_generation:
             logger.info(
                 "DBT: skipping source sync, defaulting to original file state",
-                commit_sha=self._dbt_commit_sha,
-                always_skip_envvar=os.environ.get("ALWAYS_SKIP_DBT_SYNC") == "true",
-                head_sha=os.environ.get("GITHUB_HEAD_SHA"),
+                extra={
+                    "commit_sha": self._dbt_commit_sha,
+                    "head_sha": os.environ.get("GITHUB_HEAD_SHA")
+                    or os.environ.get("BUILDKITE_COMMIT"),
+                },
             )
             projects = super().selected_projects()
             return {
                 fp: dapi
                 for project in projects
                 for fp, dapi in project.filter_dapis(self.original_file_state).items()
-            }
+            }, {}
 
-        result = {}
+        base_generated_files = {}
+        generated_metadata = {}
         for table in self._get_dbt_models():
             if table.is_allowlisted:
                 dapi_location = table.project.construct_dapi_location(table.name)
@@ -559,7 +579,7 @@ class DbtDapiValidator(DapiValidator, DBTCloudMixin):
                         os.path.dirname(dapi_location),
                     )
 
-                result[dapi_location] = {
+                base_generated_files[dapi_location] = {
                     "schema": OPENDAPI_SPEC_URL.format(
                         version=self.SPEC_VERSION,
                         entity="dapi",
@@ -574,8 +594,20 @@ class DbtDapiValidator(DapiValidator, DBTCloudMixin):
                     "primary_key": table.primary_keys,
                     "context": context,
                     "privacy_requirements": {
-                        "dsar_access_endpoint": None,
-                        "dsar_deletion_endpoint": None,
+                        "dsr_access_endpoint": None,
+                        "dsr_deletion_endpoint": None,
                     },
                 }
-        return result
+                generated_metadata[dapi_location] = {
+                    "custom_schema": table.custom_schema,
+                    "current_schema": table.schema,
+                }
+        return base_generated_files, generated_metadata
+
+    def _get_additional_metadata_from_generated(self, filepath: str) -> Optional[Dict]:
+        """Get the additional metadata from the generated"""
+        return self._base_generated_files_and_generated_metadata[1].get(filepath)
+
+    def _get_base_generated_files(self) -> Dict[str, Dict]:
+        """Get the base generated files"""
+        return self._base_generated_files_and_generated_metadata[0]
