@@ -390,7 +390,7 @@ class PrismaProjectInfo(ProjectInfo):
         return tables
 
 
-class PrismaDapiValidator(DapiValidator):
+class PrismaDapiValidator(DapiValidator[PrismaProjectInfo]):
     """
     Validator class for DAPI files created for Prisma schemas
     """
@@ -498,8 +498,8 @@ class PrismaDapiValidator(DapiValidator):
                             ),
                         },
                         "privacy_requirements": {
-                            "dsar_access_endpoint": None,
-                            "dsar_deletion_endpoint": None,
+                            "dsr_access_endpoint": None,
+                            "dsr_deletion_endpoint": None,
                         },
                     }
         return result
