@@ -898,8 +898,8 @@ class TypeOrmDapiValidator(PackageScopedDapiValidatorBase):
                     "rel_model_path": os.path.relpath(table.filename, dapi_location),
                 },
                 "privacy_requirements": {
-                    "dsar_access_endpoint": None,
-                    "dsar_deletion_endpoint": None,
+                    "dsr_access_endpoint": None,
+                    "dsr_deletion_endpoint": None,
                 },
             }
 
